/*
 * Network.cpp
 *
 *  Created on: Feb 4, 2015
 *      Author: rmin
 */

#include "Network.h"
#include <chrono>
#include "Controller.h"

Network::Network(void* controller) {
// TODO Auto-generated constructor stub
	this->controllerVoid = controller;
	io_service = NULL;
	resolver = NULL;
	query = NULL;
	socket = NULL;
	isConnected = false;
	isTerminated = false;
}

void Network::setConnectionData(std::string host, std::string port,
		std::string token) {
	this->host = host;
	this->port = port;
	this->token = token;
}

void Network::connect() {
	Message msg;
	try {
		isConnected = false;
		io_service = new boost::asio::io_service();

		resolver = new tcp::resolver(*io_service);
		query = new tcp::resolver::query(host, port);
		endpoint_iterator = resolver->resolve(*query);

		socket = new tcp::socket(*io_service);
		boost::system::error_code error;
		boost::asio::connect(*socket, endpoint_iterator, error);
		if (error) {
			throw boost::system::system_error(error);
		}

		Message msg;
		msg.addNode("name", "token");
		std::vector<std::string> tmp;
		tmp.push_back(token);
		msg.addArray("args", tmp);
		send(msg);

//		boost::array<char,1000> buf;
//		std::cerr << "START OF READ SOME\n";
//		socket->read_some(boost::asio::buffer(buf), error);
//		std::cerr << "END OF READ SOME\n";
//		std::string tmp(buf.elems);
//		msg.setJson(tmp);
//		std::cerr << msg.getName() << std::endl;
//		if(msg.getName() != "init")
//		{
//			terminate();
//			return;
//		}
//		if (error == boost::asio::error::eof) {
//			throw boost::asio::error::connection_aborted;
//		} else if (error)
//			throw boost::system::system_error(error); // Some other error.

	} catch (std::exception& e) {
		std::cerr << e.what() << std::endl;
		return;
	}
	isConnected = true;
	isTerminated = false;
	startReceiving();
}

void Network::startReceiving() {
	while (isConnected && !isTerminated) {
		if (!doReceive())
			break;
	}
	terminate();
}

bool Network::doReceive() {
	try {
		boost::system::error_code error;
		boost::array<char, 1000 * 1000> buf;
		int sizeMessage = (int) socket->read_some(boost::asio::buffer(buf),
				error);

		SubPacket *cache = new SubPacket(MAX_LEN_OF_TCP);
		Packet.push_back(cache);
		for (int i = 0; i < sizeMessage; i++)
			if (buf.elems[i] != '\0') {
				Packet[Packet.size() - 1]->push(buf.elems[i]);
			} else {
				parse();
				Packet.push_back(new SubPacket(MAX_LEN_OF_TCP));
			}
		if (error != 0)
			return false;
	} catch (std::exception& e) {
		std::cerr << e.what() << std::endl;
		return false;
	}
	return true;
}

void Network::parse() {

	std::string str;
	for (int i = 0; i < (int) Packet.size(); i++) {
		str.append(Packet[i]->buffer.begin(),
				Packet[i]->buffer.begin() + Packet[i]->index);
	}
//	str.push_back('\0');
	Message msg;
	msg.setJson(str);

	Controller* controller = (Controller*) controllerVoid;
	controller->handleMessage(msg);
	clearPacket();
}

void Network::clearPacket() {
	for (int i = 0; i < (int) Packet.size(); i++)
		delete Packet[i];
	Packet.clear();
}

void Network::send(Message msg) {
	while (true) {
		boost::system::error_code error;
//		std::cerr << "Msg Size : " << msg.getJson().size() << std::endl;
		socket->write_some(boost::asio::buffer(msg.getJson()), error);
		if (error == boost::asio::error::eof)
			throw "Connection closed by peer";
		if (error == 0) {
			break;
		}
	}
}

void Network::terminate() {
	isTerminated = true;
	isConnected = false;
	if (socket != NULL)
		socket->close();
	if (io_service != NULL) {
		delete io_service;
		io_service = NULL;
	}
	if (resolver != NULL) {
		delete resolver;
		resolver = NULL;
	}
	if (query != NULL) {
		delete query;
		query = NULL;
	}
	if (socket != NULL) {
		delete socket;
		socket = NULL;
	}
}

Network::~Network() {
	terminate();
//	if (socket)
//		socket->close();
//	if (io_service)
//		delete io_service;
//	if (resolver)
//		delete resolver;
//	if (query)
//		delete query;
//	if (socket)
//		delete socket;
// TODO Auto-generated destructor stub
}

bool Network::getIsConnected() {
	return isConnected;
}

bool Network::getIsTerminated() {
	return isTerminated;
}
