/*
 * World.cpp
 *
 *  Created on: Feb 15, 2015
 *      Author: rmin
 */

#include "World.h"

const std::string BLOCK_TYPE_NONE = "none";
const std::string BLOCK_TYPE_NORMAL = "normal";
const std::string BLOCK_TYPE_MITOSIS = "mitosis";
const std::string BLOCK_TYPE_RESOURCE = "resource";
const std::string BLOCK_TYPE_IMPASSAblE = "impassable";

World::World(EventHandler* eventHandler, ClientInitInfo *initInfo, Map *map) {
	// TODO Auto-generated constructor stub
	this->eventHandler = eventHandler;
	this->teams = initInfo->getTeams();
	this->myId = initInfo->getYourInfo().getId();
	this->myName = initInfo->getYourInfo().getName();
	this->mapSize = initInfo->getMapSize();
	this->map = map;
	this->turn = initInfo->getTurn();
}

World::~World() {
	// TODO Auto-generated destructor stub
	delete map;
}

std::vector<std::string>& World::getTeams() {
	return teams;
}

int World::getMyId() {
	return myId;
}

std::string World::getMyName() {
	return myName;
}

MapSize World::getMapSize() {
	return mapSize;
}

Map* World::getMap() {
	return map;
}

int World::getTurn() {
	return turn;
}

void World::setTurn(int turn) {
	this->turn = turn;
}

void World::clearDynamics() {
	allCells.clear();
	myCells.clear();
	enemyCells.clear();
}

void World::addCell(DynamicData dynamicData) {
	Cell *cell = new Cell(eventHandler, dynamicData);
	allCells.push_back(cell);
	if (dynamicData.getOther().getAsInt("teamId") == myId) {
		myCells.push_back(cell);
	} else {
		enemyCells.push_back(cell);
	}
}

std::vector<Cell*> World::getAllCells() {
	return allCells;
}

std::vector<Cell*> World::getMyCells() {
	return myCells;
}

std::vector<Cell*> World::getEnemyCells() {
	return enemyCells;
}

void World::setStaticChange(StaticData s) {
	if (s.getType() == BLOCK_TYPE_NONE || s.getType() == BLOCK_TYPE_NORMAL
			|| s.getType() == BLOCK_TYPE_MITOSIS
			|| s.getType() == BLOCK_TYPE_RESOURCE
			|| s.getType() == BLOCK_TYPE_IMPASSAblE)
		map->setChange(s);
}

void World::clearDynamic()
{
	for(int i=0;i<(int)allCells.size();i++)
	{
		delete allCells[i];
	}
	allCells.clear();
	myCells.clear();
	enemyCells.clear();
}
